@@ -18,21 +18,21 @@
 #include "index_list.hpp"
 
 
-#define PY_MEM_NEW_DELETE void *operator new(size_t s) {            \
+#define PY_MEM_NEW_DELETE static void *operator new(size_t s) {     \
         void *ptr = PyObject_Malloc(s);                             \
         if(!ptr) throw std::bad_alloc();                            \
         return ptr;                                                 \
     }                                                               \
-    void operator delete(void *ptr) {                               \
+    static void operator delete(void *ptr) {                        \
         PyObject_Free(ptr);                                         \
     }
 
-#define PY_MEM_GC_NEW_DELETE void *operator new(size_t s) {         \
+#define PY_MEM_GC_NEW_DELETE static void *operator new(size_t s) {  \
         void *ptr = _PyObject_GC_Malloc(s);                         \
         if(!ptr) throw std::bad_alloc();                            \
         return ptr;                                                 \
     }                                                               \
-    void operator delete(void *ptr) {                               \
+    static void operator delete(void *ptr) {                        \
         PyObject_GC_Del(ptr);                                       \
     }
 
@@ -44,7 +44,7 @@
 #if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION == 3
     // TODO: find the earliest version this was defined in
     #define PyObject_LengthHint _PyObject_LengthHint
-#elif PY_MAJOR_VERSION < 3 || (PY_MARJOR_VERSION == 3 && PY_MINOR_VERSION < 3)
+#elif PY_MAJOR_VERSION < 3 || (PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION < 3)
     Py_ssize_t PyObject_LengthHint(PyObject *o,Py_ssize_t defaultlen);
 #endif
 
@@ -209,7 +209,10 @@ inline PyObject *to_pyobject(PyObject *x) {
 }
 
 
-template<typename T> inline T from_pyobject(PyObject *o);
+template<typename T> inline T from_pyobject(PyObject *o) {
+    static_assert(sizeof(T) == 0,"no conversion to this type defined");
+    return T();
+}
 
 
 template<> inline short from_pyobject<short>(PyObject *o) {
@@ -395,7 +398,7 @@ template<typename T,typename Base,bool AllowIndirect=false,bool InPlace=(alignof
 template<typename T,typename Base> struct simple_py_wrapper<T,Base,false,false> : Base {
     T *base;
     PY_MEM_NEW_DELETE
-    template<typename... Args> simple_py_wrapper(Args&&... args) {
+    template<typename... Args> simple_py_wrapper(Args&&... args) : base(nullptr) {
         PyObject_Init(reinterpret_cast<PyObject*>(this),Base::pytype());
         new(&alloc_base()) T(std::forward<Args>(args)...);
     }
@@ -526,6 +529,10 @@ template<typename T> struct opt_parameter {
 };
 
 class get_arg {
+public:
+    enum arg_type {REQUIRED,OPTIONAL,KEYWORD_ONLY};
+
+private:
     struct get_arg_base {
         PyObject *args, *kwds;
         const char **names;
@@ -534,16 +541,16 @@ class get_arg {
         
         get_arg_base(PyObject *args,PyObject *kwds,Py_ssize_t arg_len,const char **names,const char *fname);
         
-        PyObject *operator()(size_t i,bool required);
+        PyObject *operator()(size_t i,arg_type type);
         void finished();
     } base;
     
     template<typename T> static inline T as_param(const parameter<T> &p,size_t index,get_arg_base &ga) {
-        return from_pyobject<T>(ga(index,true));
+        return from_pyobject<T>(ga(index,REQUIRED));
     }
     
     template<typename T> static inline T as_param(const opt_parameter<T> &p,size_t index,get_arg_base &ga) {
-        PyObject *tmp = ga(index,false);
+        PyObject *tmp = ga(index,OPTIONAL);
         return tmp ? from_pyobject<T>(tmp) : p.def_value;
     }
     
@@ -558,10 +565,11 @@ class get_arg {
 public:
     Py_ssize_t arg_index;
     
-    get_arg(PyObject *args,PyObject *kwds,Py_ssize_t arg_len,const char **names=NULL,const char *fname=NULL) : base(args,kwds,arg_len,names,fname), arg_index(0) {}
-    template<int N> get_arg(PyObject *args,PyObject *kwds,const char *(&names)[N],const char *fname=NULL) : get_arg(args,kwds,N,names,fname) {}
+    get_arg(PyObject *args,PyObject *kwds,Py_ssize_t arg_len,const char **names=NULL,const char *fname=nullptr) : base(args,kwds,arg_len,names,fname), arg_index(0) {}
+    template<int N> get_arg(PyObject *args,PyObject *kwds,const char *(&names)[N],const char *fname=nullptr) : get_arg(args,kwds,N,names,fname) {}
     
-    PyObject *operator()(bool required) { return base(arg_index++,required); }
+    PyObject *operator()(bool required) { return base(arg_index++,required ? REQUIRED : OPTIONAL); }
+    PyObject *operator()(arg_type type) { return base(arg_index++,type); }
     void finished() { base.finished(); }
     
     template<typename... P> static std::tuple<typename P::type...> get_args(const char *fname,PyObject *args,PyObject *kwds,const P&... params) {
@@ -569,10 +577,8 @@ public:
     }
 };
 
-constexpr parameter<PyObject*> param(const char *name) { return parameter<PyObject*>(name); }
-template<typename T> constexpr parameter<T> param(const char *name) { return parameter<T>(name); }
-constexpr opt_parameter<PyObject*> param(const char *name,PyObject *def_value) { return {name,def_value}; }
-template<typename T> constexpr opt_parameter<T> param(const char *name,T def_value) { return {name,def_value}; }
+template<typename T=PyObject*> constexpr parameter<T> param(const char *name) { return parameter<T>(name); }
+template<typename T=PyObject*> constexpr opt_parameter<T> param(const char *name,T def_value) { return {name,def_value}; }
 
 
 void NoSuchOverload(PyObject *args);
