@@ -5,6 +5,10 @@ tedious to do by hand"""
 
 from __future__ import print_function
 
+import os
+import errno
+import os.path
+import tempfile
 import json
 import re
 
@@ -54,6 +58,14 @@ reduce_meth = """        FORCE_INLINE {item_type} {extra}() const {{
             return {intr}(this->data.p);
         }}
 """
+mask_meth = """        FORCE_INLINE {wrap_type} {extra}(mask m) const {{
+            return {intr}(m.data,this->data.p);
+        }}
+"""
+pseudo_maskz_move = """        FORCE_INLINE {wrap_type} {extra}(mask m) const {{
+            return {intr}(reinterpret_cast<{base_type}>(m.data),this->data.p);
+        }}
+"""
 
 have_vec_reduce_add = '        static constexpr bool has_vec_reduce_add = true;\n'
 reduce_add_meth = reduce_meth + have_vec_reduce_add
@@ -112,8 +124,12 @@ cmp_func = """    FORCE_INLINE {wrap_type}::mask {extra[0]}({wrap_type} a,{wrap_
     }}
 """
 
-mask_func = """    FORCE_INLINE mask{ssize}_v_{psize} {extra}({args_a}) {{
-        return
+testz_func = """    FORCE_INLINE int {extra}({wrap_type} a,{wrap_type} b) {{
+        return {intr}(a.data.p,b.data.p);
+    }}
+    FORCE_INLINE int {extra}({wrap_type} a) {{
+        return {intr}(a.data.p,a.data.p);
+    }}
 """
 
 def print_mask_function(output,swidth,pwidth,func,args,body,fallback,explicit_cast=True):
@@ -282,6 +298,56 @@ class RepeatTransform(FTransform):
             r += 'x'
         return r
 
+class BlendTransformBase(FTransform):
+    def __init__(self):
+        super(BlendTransformBase,self).__init__(None,self.template)
+    
+    def intrinsic(self,type,width):
+        base = 'mask_blend'
+        if width < 512:
+            base = 'blendv'
+            if type.float == INTEGER:
+                # The only integer versions of the blendv instructions operate
+                # on 8bit types, but as long as each value in the mask is all
+                # ones or all zeros, the result is the same for any integer
+                # size.
+                type = types['int8_t']
+        return generic_intrinsic_name(type,width,base)
+    
+    def output(self,type,width,intr):
+        params = self.base_params
+        mparam = 'm.data'
+        if type.float == INTEGER and type.size >= 32:
+            mparam = 'reinterpret_cast<{0}>({1})'.format(base_type(type,width),mparam)
+        
+        if width < 512:
+            params += ',' + mparam
+        else:
+            params = mparam + ',' + params
+        
+        return self.template.format(
+            type.size,
+            width,
+            wrap_type(type,width),
+            intr['name'],
+            params)
+
+class MaskSetTransform(BlendTransformBase):
+    base_params = 'this->data.p,b.data.p'
+    template = '''        FORCE_INLINE {2} &mask_set(mask{0}_v_{1} m,{2} b) {{
+            this->data.p = {3}({4});
+            return *this;
+        }}
+'''
+
+class BlendTransform(BlendTransformBase):
+    # parameters are reversed to match "A ? B : C" syntax
+    base_params = 'b.data.p,a.data.p'
+    template = '''    FORCE_INLINE {2} mask_blend(mask{0}_v_{1} m,{2} a,{2} b) {{
+        return {3}({4});
+    }}
+'''
+
 class CastTransform(object):
     def supported(self,type,width,src_width,inline):
         return not (width == src_width or
@@ -328,6 +394,7 @@ mixed_method_transforms = [
 
 reduce_add = FTransform('reduce_add',reduce_add_meth)
 repeat_a = BroadcastTransform()
+maskz_mov = FTransform('maskz_mov',mask_meth,'zfilter')
 
 method_transforms = [MixedFTransformAdapter(w,mmt) for w in widths for mmt in mixed_method_transforms] + [
     FTransform('add',binary_op,'+'),
@@ -359,25 +426,34 @@ method_transforms = [MixedFTransformAdapter(w,mmt) for w in widths for mmt in mi
     repeat_a,
     ShuffleTransform(prefer=repeat_a),
     RepeatTransform('set1',set1_meth,'repeat'),
-    AltIFTransform('setzero',setzero_meth,'zeros')]
+    maskz_mov,
+    FTransform('and',pseudo_maskz_move,'zfilter',prefer=maskz_mov),
+    
+    # if maskz_mov is available, use the fallback nzfilter (see simd.hpp.in)
+    # method instead this
+    FTransform('andnot',pseudo_maskz_move,'nzfilter',prefer=maskz_mov),
+    
+    AltIFTransform('setzero',setzero_meth,'zeros'),
+    MaskSetTransform()]
 
 function_transforms = [
     FTransform('max',binary_func),
     FTransform('min',binary_func),
     AltIFTransform('andnot',binary_func,'and_not'),
+    AltIFTransform('testz',testz_func,'test_z'),
     CmpFTransform('cmp_nlt',cmp_func,'nlt'),
     CmpFTransform('cmp_nle',cmp_func,'nle'),
     CmpFTransform('cmp_ngt',cmp_func,'ngt'),
-    CmpFTransform('cmp_nge',cmp_func,'nge')]
+    CmpFTransform('cmp_nge',cmp_func,'nge'),
+    BlendTransform()]
 
 
 def data_args(args):
     return ['{0}.data'.format(chr(ord('a')+i)) for i in range(args)]
 
 def ideal_args(swidth,pwidth,args):
-    r = data_args(args)
-    if swidth >= 32: r = ['reinterpret_cast<__m{0}i>({1})'.format(pwidth,a) for a in r]
-    return ','.join(r)
+    if swidth >= 32: args = ['reinterpret_cast<__m{0}i>({1})'.format(pwidth,a) for a in args]
+    return ','.join(args)
 
 def avx512_mask_body(swidth,base,args):
     r = '_mm512_k{0}({1})'.format(base,','.join(data_args(args)))
@@ -392,7 +468,7 @@ def common_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix,name,in
     else:
         intr = intrinsics['{0}_{1}_si{2}'.format(mm_prefix(pwidth),intr_base,pwidth)]
 
-        body = '{0}_{1}_si{2}({3})'.format(mm_prefix(pwidth),intr_base,pwidth,ideal_args(swidth,pwidth,2))
+        body = '{0}_{1}_si{2}({3})'.format(mm_prefix(pwidth),intr_base,pwidth,ideal_args(swidth,pwidth,data_args(2)))
         
         if intr['tech'] not in IMPLIED_REQ[type_req]:
             assert swidth >= 32
@@ -418,7 +494,7 @@ def not_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix):
             mm_prefix(pwidth),
             ['','eq'][in_name],
             swidth,
-            ideal_args(swidth,pwidth,1),
+            ideal_args(swidth,pwidth,data_args(1)),
             pwidth,
             [',_CMP_EQ_UQ',''][in_name])
         
@@ -445,7 +521,7 @@ def xnor_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix):
             pwidth,
             mm_prefix(pwidth),
             ['','eq'][in_name],
-            ideal_args(swidth,pwidth,2),
+            ideal_args(swidth,pwidth,data_args(2)),
             [',_CMP_EQ_UQ',''][in_name])
         
         if intr['tech'] not in IMPLIED_REQ[type_req]:
@@ -454,6 +530,53 @@ def xnor_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix):
         
     print_mask_function(output,swidth,pwidth,'l_xnor',2,body,fallback,pwidth == 512)
 
+def mask_method_body(intr,intr_base,swidth,pwidth,type_req,args,suffix):
+    body = '{0}({1})'.format(intr['name'],ideal_args(swidth,pwidth,args))
+    
+    if intr['tech'] not in IMPLIED_REQ[type_req]:
+        assert swidth >= 32
+        body = '''
+    #ifdef SUPPORT_{0}
+                {1}
+    #else
+                {2}_{3}_{4}({5})
+    #endif
+                '''.format(macroize(intr['tech']),body,mm_prefix(pwidth),intr_base,suffix,','.join(args))
+
+    return body
+
+def setzero_mask_method(output,intrinsics,swidth,pwidth,type_req,suffix):
+    print('        static FORCE_INLINE mask{0}_v_{1} zeros() {{'.format(swidth,pwidth),sep='',end='',file=output)
+    
+    if pwidth == 512:
+        print(' return mask{0}_v_{1}(0); '.format(swidth,pwidth),sep='',end='',file=output)
+    else:
+        intr = intrinsics['{0}_setzero_si{1}'.format(mm_prefix(pwidth),pwidth)]
+    
+        print(
+            '\n            return mask{0}_v_{1}('.format(swidth,pwidth),
+            mask_method_body(intr,'setzero',swidth,pwidth,type_req,[],suffix),
+            ');\n        ',
+            sep='',end='',file=output)
+
+    print('}\n',sep='',file=output)
+
+def any_mask_method(output,intrinsics,swidth,pwidth,type_req,suffix):
+    # the default method is fine for AVX512
+    if pwidth != 512:
+        intr = intrinsics['{0}_testz_si{1}'.format(mm_prefix(pwidth),pwidth)]
+        always = intr['tech'] in IMPLIED_REQ[type_req]
+        
+        if not always:
+            print(ifdef_support(intr['tech'],indent=1),sep='',file=output)
+            type_req = intr['tech']
+        
+        print(
+            '        FORCE_INLINE bool any() const {{\n            return {0} == 0;\n        }}'.format(
+                mask_method_body(intr,'testz',swidth,pwidth,type_req,['data','data'],suffix)),
+            sep='',file=output)
+        if not always: print('    #endif\n',sep='',file=output)
+
 
 def macroize(x):
     return ''.join(c if c.isalnum() else '_' for c in x)
@@ -499,7 +622,7 @@ del rs
 
 meth_search = re.compile(r'^\s*//\s*\[\[\[(\w+)-(\d+)(?:-(\d+)|)\]\]\]\s*$')
 func_search = re.compile(r'^\s*//\s*\[\[\[functions\]\]\]\s*$')
-mask_search = re.compile(r'^\s*//\s*\[\[\[mask-(\d+)-(\d+)\]\]\]\s*$')
+mask_search = re.compile(r'^\s*//\s*\[\[\[mask-(\d+)-(\d+)(-outer|)\]\]\]\s*$')
 
 
 def generate(data_file,template_file,output_file):
@@ -514,109 +637,125 @@ def generate(data_file,template_file,output_file):
 
 
     template = open(template_file,'r')
-    output = open(output_file,'w')
+    output = tempfile.NamedTemporaryFile(mode='w',dir=os.path.dirname(output_file),delete=False)
+    
+    try:
+        print('// THIS FILE WAS GENERATED BY generate_simd.py. DO NOT EDIT IT DIRECTLY.\n\n',file=output)
 
-    print('// THIS FILE WAS GENERATED BY generate_simd.py. DO NOT EDIT IT DIRECTLY.\n\n',file=output)
-
-    for line in template:
-        m = mask_search.match(line)
-        if m:
-            swidth = int(m.group(1),10)
-            pwidth = int(m.group(2),10)
-            
-            if pwidth == 512:
-                type_req = 'AVX-512'
-            elif pwidth == 256:
-                type_req = 'AVX2' if swidth < 32 else 'AVX'
-            else:
-                assert pwidth == 128
-                type_req = 'SSE' if swidth == 32 else 'SSE2'
-            
-            if swidth == 64:
-                suffix = 'pd'
-            elif swidth == 32:
-                suffix = 'ps'
-            else:
-                suffix = 'epi{0}'.format(swidth)
-            
-            common_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix,'operator&&','and')
-            common_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix,'operator||','or')
-            not_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix)
-            common_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix,'l_andn','andn' if pwidth == 512 else 'andnot')
-            common_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix,'l_xor','xor')
-            xnor_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix)
-            continue
-        
-        m = meth_search.match(line)
-        if m:
-            type = types[m.group(1)]
-            width = int(m.group(2),10)
-            assert width in widths
-            
-            if m.group(3):
-                width2 = int(m.group(3),10)
-                assert width2 in widths
+        for line in template:
+            outer = True
+            m = mask_search.match(line)
+            if m:
+                swidth = int(m.group(1),10)
+                pwidth = int(m.group(2),10)
+                outer = bool(m.group(3))
                 
-                for mt in mixed_method_transforms:
-                    if not mt.supported(type,width,width2,False): continue
-                
-                    intr_name = mt.intrinsic(type,width,width2,False)
-                    intr = intr_name and intrinsics.get(intr_name)
-                    if intr:
-                        print(mt.output(type,width,width2,intr,False),file=output)
-            else:
-                if width == 512:
-                    assert type.size >= 32
+                if pwidth == 512:
                     type_req = 'AVX-512'
-                elif type is types['float']:
-                    type_req = {128:'SSE',256:'AVX'}[width]
-                elif type is types['double']:
-                    type_req = {128:'SSE2',256:'AVX'}[width]
-                elif width == 256:
-                    type_req = 'AVX2'
+                elif pwidth == 256:
+                    type_req = 'AVX2' if swidth < 32 else 'AVX'
                 else:
-                    assert width == 128
-                    type_req = 'SSE2'
+                    assert pwidth == 128
+                    type_req = 'SSE' if swidth == 32 else 'SSE2'
                 
-                dependent = {}
-                for mt in method_transforms:
-                    if mt.supported(type,width):
-                        intr = intrinsics.get(mt.intrinsic(type,width))
+                if swidth == 64:
+                    suffix = 'pd'
+                elif swidth == 32:
+                    suffix = 'ps'
+                else:
+                    suffix = 'epi{0}'.format(swidth)
+                
+                if outer:
+                    common_mask_function(output.file,intrinsics,swidth,pwidth,type_req,suffix,'operator&&','and')
+                    common_mask_function(output.file,intrinsics,swidth,pwidth,type_req,suffix,'operator||','or')
+                    not_mask_function(output.file,intrinsics,swidth,pwidth,type_req,suffix)
+                    common_mask_function(output.file,intrinsics,swidth,pwidth,type_req,suffix,'l_andn','andn' if pwidth == 512 else 'andnot')
+                    common_mask_function(output.file,intrinsics,swidth,pwidth,type_req,suffix,'l_xor','xor')
+                    xnor_mask_function(output.file,intrinsics,swidth,pwidth,type_req,suffix)
+                else:
+                    setzero_mask_method(output.file,intrinsics,swidth,pwidth,type_req,suffix)
+                    any_mask_method(output.file,intrinsics,swidth,pwidth,type_req,suffix)
+                continue
+            
+            m = meth_search.match(line)
+            if m:
+                type = types[m.group(1)]
+                width = int(m.group(2),10)
+                assert width in widths
+                
+                if m.group(3):
+                    width2 = int(m.group(3),10)
+                    assert width2 in widths
+                    
+                    for mt in mixed_method_transforms:
+                        if not mt.supported(type,width,width2,False): continue
+                    
+                        intr_name = mt.intrinsic(type,width,width2,False)
+                        intr = intr_name and intrinsics.get(intr_name)
                         if intr:
-                            req = intr['tech']
-                            antireq = mt.antirequisite(type,width,intrinsics)
+                            print(mt.output(type,width,width2,intr,False),file=output.file)
+                else:
+                    if width == 512:
+                        assert type.size >= 32
+                        type_req = 'AVX-512'
+                    elif type is types['float']:
+                        type_req = {128:'SSE',256:'AVX'}[width]
+                    elif type is types['double']:
+                        type_req = {128:'SSE2',256:'AVX'}[width]
+                    elif width == 256:
+                        type_req = 'AVX2'
+                    else:
+                        assert width == 128
+                        type_req = 'SSE2'
+                    
+                    dependent = {}
+                    for mt in method_transforms:
+                        if mt.supported(type,width):
+                            intr = intrinsics.get(mt.intrinsic(type,width))
+                            if intr:
+                                req = intr['tech']
+                                antireq = mt.antirequisite(type,width,intrinsics)
 
-                            if antireq is None and req in IMPLIED_REQ[type_req]:
-                                print(mt.output(type,width,intr),file=output)
-                            elif antireq is None or antireq not in IMPLIED_REQ[type_req]:
-                                rs = (None if req in IMPLIED_REQ[type_req] else req,antireq)
+                                if antireq is None and req in IMPLIED_REQ[type_req]:
+                                    print(mt.output(type,width,intr),file=output.file)
+                                elif antireq is None or antireq not in IMPLIED_REQ[type_req]:
+                                    rs = (None if req in IMPLIED_REQ[type_req] else req,antireq)
+                                    if rs not in dependent:
+                                        dependent[rs] = []
+                                    dependent[rs].append((mt,intr,type,width))
+                            
+                    print_dependent(dependent,output.file,1)
+                continue
+
+            m = func_search.match(line)
+            if m:
+                dependent = {}
+                for type in types.values():
+                    for width in widths:
+                        for ft in function_transforms:
+                            if not ft.supported(type,width): continue
+                            
+                            intr = intrinsics.get(ft.intrinsic(type,width))
+                            if intr:
+                                rs = (intr['tech'],ft.antirequisite(type,width,intrinsics))
                                 if rs not in dependent:
                                     dependent[rs] = []
-                                dependent[rs].append((mt,intr,type,width))
-                        
-                print_dependent(dependent,output,1)
-            continue
+                                dependent[rs].append((ft,intr,type,width))
+                            
+                print_dependent(dependent,output.file)
+                continue
 
-        m = func_search.match(line)
-        if m:
-            dependent = {}
-            for type in types.values():
-                for width in widths:
-                    for ft in function_transforms:
-                        if not ft.supported(type,width): continue
-                        
-                        intr = intrinsics.get(ft.intrinsic(type,width))
-                        if intr:
-                            rs = (intr['tech'],ft.antirequisite(type,width,intrinsics))
-                            if rs not in dependent:
-                                dependent[rs] = []
-                            dependent[rs].append((ft,intr,type,width))
-                        
-            print_dependent(dependent,output)
-            continue
-
-        print(line,end='',file=output)
-                
+            print(line,end='',file=output.file)
+    except:
+        output.file.close()
+        os.remove(output.name)
+        raise
+    
     template.close()
-    output.close()
+    output.file.close()
+    try:
+        os.remove(output_file)
+    except OSError as e:
+        if e.errno != errno.ENOENT: raise
+    os.rename(output.name,output_file)
 
